{{
***************************************
* File jvmMath: performing arithmetic *
***************************************
}}

{{
  Bytecode arithmetic is performed at a very basic level. In general, values are popped off the stack, an 
  arithmetic function is performed, and the result of the operation is placed back on the stack, which 
  effectively replaces the original values. To perform the bytecode arithmetic correctly, both values must be 
  of the same type.

  Addition. 
  In bytecode addition, two values are popped off the stack and added; then the sum is placed back on the stack.  

  Subtraction. 
  In bytecode subtraction, two values are popped off the stack, the second value is subtracted from the first, 
  and the result is placed back on the stack.  

  Multiplication. 
  In bytecode multiplication, the values are popped off the stack and multiplied; then the product is placed 
  back on the stack.  

  Division.
  In bytecode division, two values are popped off the stack, the first value is divided by the second value, 
  and the quotient is placed back on the stack. The result is truncated to the nearest integer. 
  Note:
  For integers, shorts, and long integers, an attempt to divide by zero results in an ArithmeticException.
  For floating-point numbers, an attempt to divide by zero results in the quotient being not a number. If 
  you remember NAN from calculus, you'll know that it's immeasurable and will distort your results.
  The Virtual Machine can check for NAN values using bytecodes that perform comparisons.  

  Remainders. 
  To get a remainder for division operations on integers and long integers, two values are popped off the 
  stack, the first value is divided by the second value, and the remainder is placed back on the stack. The 
  result is always truncated to the nearest integer. Therefore, to get a quotient and a remainder, two division 
  operations are done by the compiler. 
  Note:
  For integers, shorts, and long integers, an attempt to divide by zero results in an ArithmeticException.  

  To get a remainder for division operations on floating-point numbers, two values are popped off the 
  stack. The first value is divided by the second value and then multiplied by the second value. The product 
  is subtracted from the first value, and the result is placed back on the stack. The result always rounds to 
  the nearest integer, with a tie going to the even number. 
  Note:
  For floating-point numbers, an attempt to divide by zero results in the quotient being not a number.

  Negation. 
  In bytecode negation, a value is popped off the stack and negated, and the result is placed back on the stack.  
}}


OBJ
  js: "jvmStack"
  jc: "jvmConvert"
  jd: "jvmData"
  
  
PUB jb_iadd | w1,w2
{{ Meaning: Integer add. }}
{{
  JEM format: iadd
}}
  w2 := js.jb_pop
  w1 := js.jb_pop
  jd.clearMathException
  w1 := ~~w1 + ~~w2 'b15 sign extended addition
  jd.clearCarry
  if w1 < -32768 OR w1 > 32767
    jd.setCarry
  js.jb_push(w1)
  jd.incJvmPC(1)
  

{{PUB jb_ladd}}
{{ Meaning: Long integer add. }}


{{PUB jb_fadd}}
{{ Meaning: Single-precision floating-point add. }}


{{PUB jb_dadd}}
{{ Meaning: Double-precision floating-point add. }}


PUB jb_isub | w1,w2
{{ Meaning: Integer subtract. }}
{{
  JEM format: isub
}}
  w2 := js.jb_pop
  w1 := js.jb_pop
  jd.clearMathException
  w1 := ~~w1 - ~~w2 'b15 sign extended subtraction
  jd.clearCarry
  if w1 < -32768 OR w1 > 32767
    jd.setCarry
  js.jb_push(w1)
  jd.incJvmPC(1)


{{PUB jb_lsub}}
{{ Meaning: Long integer subtract. }}


{{PUB jb_fsub}}
{{ Meaning: Single-precision floating-point subtract. }}


{{PUB jb_dsub}}
{{ Meaning: Double-precision floating-point subtract. }}


PUB jb_imul | w1,w2
{{ Meaning: Integer multiply. }}
{{
  JEM format: imul
}}
  w2 := js.jb_pop
  w1 := js.jb_pop
  jd.clearMathException
  w1 := ~~w1 * ~~w2 'b15 sign extended multiplication
  jd.clearCarry
  if w1 < -32768 OR w1 > 32767
    jd.setCarry
  js.jb_push(w1)
  jd.incJvmPC(1)


{{PUB jb_lmul}}
{{ Meaning: Long integer multiply. }}


{{PUB jb_fmul}}
{{ Meaning: Single-precision floating-point multiply. }}


{{PUB jb_dmul}}
{{ Meaning: Double-precision floating-point multiply. }}


PUB jb_idiv | w1,w2
{{ Meaning: Integer divide. }}
{{
  JEM format: idiv
}}
  w2 := js.jb_pop
  w1 := js.jb_pop
  jd.clearMathException
  if w2 <> 0
    w1 := ~~w1 / ~~w2 'b15 sign extended division
    jd.clearCarry
    if w1 < -32768 OR w1 > 32767
      jd.setCarry
    js.jb_push(w1)
    jd.incJvmPC(1)
  else
    jd.setMathException
    js.jb_push($FFFF) 'do we need a result value ??
    jd.incJvmPC(1)
    

{{PUB jb_ldiv}}
{{ Meaning: Long integer divide. }}


{{PUB jb_fdiv}}
{{ Meaning: Single-precision floating-point divide. }}


{{PUB jb_ddiv}}
{{ Meaning: Double-precision floating-point divide. }}


PUB jb_irem | w1,w2
{{ Meaning: Integer remainder. }}
{{
  JEM format: irem
}}
  w2 := js.jb_pop
  w1 := js.jb_pop
  jd.clearMathException
  if w2 <> 0
    w1 := ~~w1 // ~~w2 'b15 sign extended division
    jd.clearCarry
    if w1 < -32768 OR w1 > 32767
      jd.setCarry
    js.jb_push(w1)
    jd.incJvmPC(1)
  else
    jd.setMathException
    js.jb_push($FFFF) 'do we need a result value ??
    jd.incJvmPC(1)
    

{{PUB jb_lrem}}
{{ Meaning: Long integer remainder. }}


{{PUB jb_frem}}
{{ Meaning: Single-precision floating-point remainder. }}


{{PUB jb_drem}}
{{ Meaning: Double-precision floating-point remainder. }}


PUB jb_ineg | w1
{{ Meaning: Integer negate. }}
{{
  JEM format: ineg
}}
  w1 := js.jb_pop
  jd.clearMathException
  w1 ^= $FFFF
  js.jb_push(w1+1)  '16 bits negation
  jd.incJvmPC(1)
  

{{PUB jb_lneg}}
{{ Meaning: Long integer negate. }}


{{PUB jb_fneg}}
{{ Meaning: Single-precision floating-point negate. }}


{{PUB jb_dneg}}
{{ Meaning: Double-precision floating-point negate. }}

